from tqdm import tqdm
import re
import argparse
import os
from collections import Counter
from dotenv import load_dotenv
from translation_canvas.readwrite_database import write_data, read_data
from InstructScore_SEScore3.InstructScore import InstructScore
from comet import download_model, load_from_checkpoint
import json
import sys
import sacrebleu


JOBS_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), "jobs")
CUDA_PATH = os.path.join(os.path.dirname(JOBS_PATH), 'tmp', 'cuda_devices.json')

def split_sentence_with_queries(sentence, queries):
    # Create a regular expression pattern to match any of the queries
    pattern = '|'.join(re.escape(query) for query in queries)
    # Split the sentence using the pattern as the delimiter
    parts = re.split(f'({pattern})', sentence, flags=re.IGNORECASE)

    # Filter out empty strings and None values
    phrases = [part.strip() for part in parts if part]

    return phrases


def get_score(text):
    keyword = re.escape('Major/minor: Major')
    num_major = re.findall(keyword, text, re.IGNORECASE)
    num_major = len(num_major)
    keyword = re.escape('Major/minor: Minor')
    num_minor = re.findall(keyword, text, re.IGNORECASE)
    num_minor = len(num_minor)
    return max(-5 * num_major - num_minor, -25)

def process_text(text, prediction, error_type_counter):
    """
    Process the given text and extract error information.

    Args:
        text (str): The input text to process.
        prediction (str): The predicted text.
        error_type_counter (dict): A dictionary to keep track of error types and their counts.

    Returns:
        tuple: A tuple containing the following:
            - pred_render_data (dict): A dictionary mapping phrases to error information.
            - num_errors (int): The number of errors found in the text.
            - error_type_counter (dict): The updated error type counter.

    """
    # print(text)
    text = text + '\n'
    num_pattern = r'Your Translation contains (\d+) errors:'
    num_errors = re.search(num_pattern, text)
    if num_errors is None:
        num_errors = 0
    else:
        num_errors = int(num_errors.group(1))
    
    pred_render_data = {}
    if num_errors == 0:
        pred_render_data[prediction] = "None"
        return pred_render_data, 0, error_type_counter
    # Define regular expression patterns
    error_pattern = r'Error type (\d+): (.+?)\nMajor/minor: (.+?)\nError location (\d+): (.+?)\nExplanation for error \d+: (.+?)\n'

    # Use re.findall() to find all matches of the patterns in the text
    matches = re.findall(error_pattern, text)
    # print(f"Matches found: {matches}")
    # print(f"text: {text}")
    
    queries = []
    query_dict = []

    # Create a dictionary to store error information
   
    for match in matches:
        error_num = int(match[0])
        error_type = match[1]
        error_scale = match[2]
        error_location = match[4][1:-1]
        error_explanation = match[5]
        
        error_type_counter[error_type] += 1

        queries.append(error_location)
        query_dict.append({
            'error_type': error_type,
            'error_scale': error_scale,
            'error_location': error_location,
            'error_explanation': error_explanation
        })
        
    phrases = split_sentence_with_queries(prediction, queries)
    for phrase in phrases:
        query_match = False
        for i,query in enumerate(queries):
            if query.lower() in phrase.lower():
                query_match = True
                pred_render_data[phrase] = query_dict[i]
                break
        if not query_match:
            pred_render_data[phrase] = "None"
    
    return pred_render_data, num_errors, error_type_counter

def setup_instructscore(src_lang, tgt_lang):
        
    batch_size = 20

    task_type = ""
    if src_lang=='zh':
        if tgt_lang=='en':
            task_type = 'mt_zh-en'
    elif src_lang=='en':
        if tgt_lang=='es':
            task_type = 'mt_en-es'
        elif tgt_lang=='ru':
            task_type = 'mt_en-ru'
        elif tgt_lang=='de':
            task_type = 'mt_en-de'

    scorer = InstructScore(task_type=task_type, batch_size=batch_size, cache_dir=os.environ['CACHE_DIR'])
    return scorer

def calculate_system_stats(dataset, ref, src, bleu, comet, run_id):
    if bleu:
        if ref:
            references = [dataset[j]['reference'] for j in range(0, len(dataset))]
            predictions = [dataset[j]['prediction'] for j in range(0, len(dataset))]
            bleu_score = sacrebleu.corpus_bleu(predictions, [references])
            write_data(f"UPDATE runs SET bleu_score = {bleu_score.score} WHERE id = {run_id};", logging=logging)
            print(f"Set BLEU score of {run_id} to {bleu_score.score}")
    
    if comet:
        if src and ref:
            model_path = download_model("Unbabel/wmt22-comet-da", saving_directory=os.environ['CACHE_DIR'])
            model = load_from_checkpoint(model_path)
            data = [{"src": dataset[j]['source'], "mt": dataset[j]['prediction'], "ref": dataset[j]['reference']} for j in range(0, len(dataset))]
            comet_score = model.predict(data)
            write_data(f"UPDATE runs SET comet_score = {comet_score['system_score']} WHERE id = {run_id};", logging=logging)
            print(f"Set COMET score of {run_id} to {comet_score['system_score']}")
            return comet_score['scores']
    return None

def evaluate(run_name, src_lang, tgt_lang, run_id, instructscore, bleu, is_comet, logging, ref, src):
    batch_size = 20
    eval_dataset=json.load(open(os.path.join(JOBS_PATH, run_name, f"{run_name}_extracted.json"), 'r'))
    
    if instructscore or is_comet:
        if os.path.exists(CUDA_PATH):
            cuda_devices = json.load(open(CUDA_PATH, 'r'))
            os.environ['CUDA_VISIBLE_DEVICES'] = ','.join(cuda_devices)
            print(f"Using GPUs: {','.join(cuda_devices)}")
        
    comet_scores = calculate_system_stats(eval_dataset, ref, src, bleu, is_comet, run_id)
    
    if instructscore:
        scorer = setup_instructscore(src_lang, tgt_lang)
        total_errors = 0
        error_type_counter = Counter()
    
    se_score_total = 0 if instructscore else 'NULL'
        
    for i in tqdm(range(0, len(eval_dataset), batch_size)):
        eval_prediction = [eval_dataset[j]['prediction'] for j in range(i, min(i + batch_size, len(eval_dataset)))]
        
        if instructscore:
            eval_reference = [eval_dataset[j]['reference'] for j in range(i, min(i + batch_size, len(eval_dataset)))]
            batch_outputs, scores_ls = scorer.score(ref_ls=eval_reference, out_ls=eval_prediction)
            
            
        for j in range(len(eval_prediction)):
            prediction = eval_dataset[i+j]['prediction'].replace("'", "''")
            
            if ref:
                reference = eval_dataset[i+j]['reference'].replace("'", "''")
                results = read_data(f"SELECT id FROM refs WHERE source_text = '{reference}';", logging=logging)
                if results == []:
                    results = write_data(f"INSERT INTO refs (source_text, lang) VALUES ('{reference}', '{tgt_lang}'); SELECT id FROM refs ORDER BY id DESC LIMIT 1;", logging=logging)
                ref_id = results[0][0]
            else: 
                ref_id = 'NULL'
            
            if src:
                source = eval_dataset[i+j]['source'].replace("'", "''")
                results = read_data(f"SELECT id FROM src WHERE source_text = '{source}';", logging=logging)
                if results == []:
                    results = write_data(f"INSERT INTO src (source_text, lang) VALUES ('{source}', '{src_lang}'); SELECT id FROM src ORDER BY id DESC LIMIT 1;", logging=logging)
                src_id = results[0][0]
            else:
                src_id = 'NULL'
            pred_render_dict = {prediction: "None"}
            se_score = scores_ls[j] if instructscore else 'NULL'
            comet_score = round(comet_scores[j],2) if comet_scores else 'NULL'
            num_errors = 'NULL'
            if instructscore:
                return_vals = process_text(batch_outputs[j], prediction, error_type_counter)
                pred_render_dict, num_errors, error_type_counter = return_vals
                total_errors += num_errors
                se_score_total += scores_ls[j]
            
            pred_id = write_data(f"INSERT INTO preds (se_score, comet_score, source_text, num_errors, src_id, ref_id, run_id) VALUES ({se_score}, {comet_score}, '{prediction}', {num_errors}, {src_id}, {ref_id}, {run_id}); SELECT id FROM preds ORDER BY id DESC LIMIT 1;", logging=logging)[0][0]
            
            for pred in pred_render_dict:
                pred_source_text = pred.replace("'", "''")
                if 'error_type' in pred_render_dict[pred]:
                    error_type = pred_render_dict[pred]['error_type'].replace("'", "''")
                    error_scale = pred_render_dict[pred]['error_scale']
                    error_explanation = pred_render_dict[pred]['error_explanation'].replace("'", "''")
                    results = write_data(f"INSERT INTO preds_text (source_text, error_type, error_scale, error_explanation, pred_id) VALUES ('{pred_source_text}', '{error_type}', '{error_scale}', '{error_explanation}', {pred_id});", logging=logging)
                else:
                    results = write_data(f"INSERT INTO preds_text (source_text, pred_id) VALUES ('{pred_source_text}', {pred_id});", logging=logging)
            
            
            total_evaluations = i + batch_size if i + batch_size < len(eval_dataset) else len(eval_dataset)
            final_se_score = se_score_total/total_evaluations if instructscore else 'NULL'
            results = write_data(f"UPDATE runs SET se_score = {final_se_score}, num_predictions = {total_evaluations}, in_progress = {total_evaluations/len(eval_dataset)} WHERE id = {run_id};", logging=logging)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--run_name', type=str, default=0)
    parser.add_argument('--src_lang', type=str, default=0)
    parser.add_argument('--tgt_lang', type=str, default=0)
    parser.add_argument('--run_id', type=int, default=0)
    parser.add_argument('--instructscore', type=bool, default=False)
    parser.add_argument('--bleu', type=bool, default=False)
    parser.add_argument('--comet', type=bool, default=False)
    parser.add_argument('--ref', type=bool, default=False)
    parser.add_argument('--src', type=bool, default=False)
    args = parser.parse_args()
    
    load_dotenv()
    
    logging = False
    
    run_name = args.run_name
    src_lang = args.src_lang
    tgt_lang = args.tgt_lang
    run_id = args.run_id
    instructscore = args.instructscore
    bleu = args.bleu
    is_comet = args.comet
    ref = args.ref
    src = args.src
    
    out_file = os.path.join(JOBS_PATH, run_name, f"{run_name}_out.txt")
    err_file = os.path.join(JOBS_PATH, run_name, f"{run_name}_err.txt")
    sys.stdout = open(out_file, 'w')
    sys.stderr = open(err_file, 'w')
    
    print(f"Running evaluation for {run_name} with src_lang={src_lang}, tgt_lang={tgt_lang}, run_id={run_id}, instructscore={args.instructscore}, bleu={args.bleu}, comet={args.comet}, ref={args.ref}, src={args.src}")

    evaluate(run_name, src_lang, tgt_lang, run_id, instructscore, bleu, is_comet, logging, ref, src)
    

        






from peft import get_peft_model, LoraConfig, TaskType
from torch.nn.parallel import DistributedDataParallel as DDP
import torch.distributed as dist
import click
import torch
import wandb
import random
from torch.utils.data import DataLoader
from datasets import Dataset
import os
from transformers import (
    AutoTokenizer,
    AutoModelForSeq2SeqLM,
    DataCollatorWithPadding,
    get_linear_schedule_with_warmup,
    AdamW
)
import math
import time
import json

# needs to adjust corresponding FLAN-T5
class exp_config():
    max_length = 1024
    lr = 0.0001
    gradient_accumulation_steps=6
    temp = 0.1
    drop_out=0.1
    activation="Tanh"
    final_activation=None
    vali_size=200

# process the data so that input and output data can be segmented
def prepare_data(inp_lines, out_lines):
    in_str = ''
    for line in inp_lines:
        in_str+=line
    out_str = ''
    for line in out_lines:
        out_str+=line
    in_ls, out_ls = in_str.split('[SEP]')[:-1], out_str.split('[SEP]')[:-1]
    in_ls = [ele[:-1] for ele in in_ls]
    out_ls = [ele[:-1] for ele in out_ls]
    return in_ls, out_ls 

def preprocess_data(in_ls, out_ls, tokenizer, max_length, batch_size, mode="train", shuffle=True, sampler=True):
    if mode=="train":
        ds = Dataset.from_dict({"input": in_ls, 'output': out_ls})
    else:
        ds = Dataset.from_dict({"input": in_ls})

    def preprocess_function(examples):
        model_data = {}
        # model inputs
        inputs = tokenizer(examples['input'], max_length=max_length, padding='max_length', truncation=True)
        model_data['input_ids'], model_data['attn_masks'] = inputs['input_ids'], inputs['attention_mask']
        # model ground truth outputs
        if mode == "train":
            outputs = tokenizer(examples['output'], max_length=max_length, padding='max_length', truncation=True)
            model_data['output_ids'] = outputs['input_ids']
        return model_data

    processed_datasets = ds.map(
        preprocess_function,
        batched=True,
        num_proc=None,
        remove_columns=ds.column_names,
        desc="Running tokenizer on dataset",
    )

    data_collator = DataCollatorWithPadding(
        tokenizer=tokenizer,
        padding='max_length',
        max_length=max_length,
        pad_to_multiple_of=None,
        return_tensors = 'pt'
    )

    if sampler:
        data_sampler = torch.utils.data.distributed.DistributedSampler(processed_datasets, shuffle=shuffle)
        dataloader = DataLoader(processed_datasets, batch_size=batch_size, collate_fn=data_collator, sampler=data_sampler)
    else:
        dataloader = DataLoader(processed_datasets, batch_size=batch_size, collate_fn=data_collator, shuffle=shuffle)
    return dataloader

@click.command()
@click.option('-flan_type')
@click.option('-peft', default=None)
@click.option('-inf')
@click.option('-outf')
@click.option('-num_epoch', default=1, type=int)
@click.option('-batch_size', default=6, type=int)
@click.option('-save_dir_name', default='flanT5_weights', type=str)
@click.option('-eval_step', type=int)
@click.option('-inference', type=bool, default=False)
def main(flan_type, peft, inf, outf, num_epoch, batch_size, save_dir_name, eval_step, inference):
    model_name_or_path = f"google/flan-t5-{flan_type}"
    tokenizer_name_or_path = f"google/flan-t5-{flan_type}"

    model = AutoModelForSeq2SeqLM.from_pretrained(model_name_or_path)
    tokenizer = AutoTokenizer.from_pretrained(tokenizer_name_or_path)

    # enable parameter efficient weight updates
    if peft == 'lora':
        peft_config = LoraConfig(
            task_type=TaskType.SEQ_2_SEQ_LM, inference_mode=False, r=8, lora_alpha=32, lora_dropout=0.1
        )
        model = get_peft_model(model, peft_config)

    # initalize the process
    dist.init_process_group(backend="nccl")
    rank = int(os.environ['LOCAL_RANK'])
    # only main process initalize wandb
    if rank == 0:
        # initalize the project parameters into Wandb, store experiment specific parameters
        wandb.init(project="COTScore", config=
        {   
            "model size": flan_type,
            "strategy": "Seq-to-Seq",
            "epoch": num_epoch,
            "eval_step": eval_step,
            "train batch size": batch_size * exp_config.gradient_accumulation_steps * 8,
            "lr": exp_config.lr,
        })

    exp_config.device_id = rank % torch.cuda.device_count()
    # set cuda device with rank and clear ram cache to ensure balanced ram allocations
    torch.cuda.set_device(rank)
    torch.cuda.empty_cache()
    model.to(exp_config.device_id)
    # parallelize the pipeline into multiple gpus
    optimizer = AdamW(model.parameters(), lr=exp_config.lr)
    # loaded data in segments
    data = json.load(open('filter_ref_based_data_april_6.json'))
    train_in_ls, train_out_ls = [], []
    for _, ele in data.items():
        train_in_ls+=[ele['input']]
        train_out_ls+=[ele['output']]

    # vali_in_ls, vali_out_ls = in_ls[:exp_config.vali_size], out_ls[:exp_config.vali_size]
    # print("Vali Inputs: ", len(vali_in_ls))
    # print("Vali Outputs: ", len(vali_out_ls))
    # train_in_ls, train_out_ls = in_ls[exp_config.vali_size:], out_ls[exp_config.vali_size:]
    # print("Train Inputs: ", len(train_in_ls))
    # print("Train Outputs: ", len(train_out_ls))
    # print(train_in_ls[1])
    # print(train_out_ls[1])

    # needs sample and shuffle for training
    train_dataloader = preprocess_data(train_in_ls, train_out_ls, tokenizer, exp_config.max_length, batch_size, mode="train", shuffle=True, sampler=True)
    # vali_dataloader = preprocess_data(vali_in_ls, vali_out_ls, tokenizer, exp_config.max_length, batch_size, mode="train", shuffle=False, sampler=False)
    # if inference:
    #     # load in testing dataset
    #     pre_test_in_ls, pre_test_out_ls = open(ein, 'r').readlines(), open(eout, 'r').readlines()
    #     test_in_ls, test_out_ls = prepare_data(pre_test_in_ls, pre_test_out_ls)
    #     test_dataloader = preprocess_data(test_in_ls, test_out_ls, tokenizer, exp_config.max_length, batch_size, mode="test", shuffle=False, sampler=False)
    
    model = DDP(model, device_ids=[exp_config.device_id], find_unused_parameters=True)
    model.train()

    max_train_steps = math.ceil(len(train_dataloader) / exp_config.gradient_accumulation_steps) * num_epoch
    num_warmup_steps=0
    lr_scheduler = get_linear_schedule_with_warmup(
        optimizer=optimizer,
        num_warmup_steps=num_warmup_steps,
        num_training_steps=max_train_steps,
    )

    # save at end of epoch and at main processls
    if not os.path.isdir(f'{save_dir_name}') and rank == 0:
        os.makedirs(f'{save_dir_name}')
        print(f'{save_dir_name} is created!')

    global_step = 0 

    with torch.autograd.set_detect_anomaly(True):
        for epoch in range(num_epoch):
            # reset the best correlation each time at start of epoch
            # cur_best_vali = float('inf')
            torch.cuda.empty_cache() # empty cache in gpus
            train_dataloader.sampler.set_epoch(epoch) # set the sampler at each epoch
            for step, train_batch in enumerate(train_dataloader):
                # accumulate losses at weights, each is normalized by accumulation steps
                train_outputs = model(input_ids=train_batch['input_ids'], attention_mask=train_batch['attn_masks'], labels=train_batch['output_ids'])
                train_loss = torch.mean(train_outputs.loss) / exp_config.gradient_accumulation_steps

                # evaluate at every eval_step and also at the end of epoch (includes the beginning loss)
                if ((step % (eval_step * exp_config.gradient_accumulation_steps) == 0) or (step == len(train_dataloader) - 1)) and rank == 0:
                    print("start to evaluate!")
                     # store all the losses in wandb
                    wandb_temp_dict = {}
                    # evaluate on the validation loss 
                    # start_eval_time = time.time()
                    # model.eval()

                    # vali_loss, count = 0, 0, 
                    # # vali_txt_ls = [] 
                    # # evaluate validation loss here
                    # with torch.no_grad():
                    #     for vali_batch in vali_dataloader:
                    #         vali_outputs=model(input_ids=vali_batch['input_ids'], attention_mask=vali_batch['attn_masks'], labels=vali_batch['output_ids'])
                    #         vali_loss+=torch.mean(vali_outputs.loss)
                    #         count+=1 
                    #         # vali_txt_ls += hypotheses_batch
                    #     vali_gen = model.module.generate(input_ids=vali_batch['input_ids'].to(exp_config.device_id), \
                    #                     attention_mask=vali_batch['attn_masks'].to(exp_config.device_id), max_new_tokens=exp_config.max_length)
                    #     hypotheses_batch = tokenizer.batch_decode(vali_gen, skip_special_tokens=True, clean_up_tokenization_spaces=True)
                    #     print(hypotheses_batch)
                    #     vali_loss /= count 

                    #vali_txt_ls = [vali+'[SEP]' for vali in vali_txt_ls]

                    print("Train Loss: ", train_loss.item())
                    # print("Vali Loss: ", vali_loss.item())
                    # wandb_temp_dict.update({"Vali Loss": vali_loss.item()})
                    wandb_temp_dict.update({"Train Loss": train_loss.item()})

                    # save at the best epoch step
                    if step == len(train_dataloader) - 1:
                        # cur_best_vali=vali_loss
                        # saveValiFile = open(f'{save_dir_name}/epoch{epoch}_step{step}.txt', 'w')
                        # saveValiFile.writelines(vali_txt_ls)
                        # print("File is saved!")
                        torch.save(model.module, f'{save_dir_name}/epoch{epoch}_end.ckpt')
                        print("Weight is saved!")

                    model.train()
                    torch.cuda.empty_cache()
                    wandb.log(wandb_temp_dict)
                    # print("Testing Duration: ", time.time()-start_eval_time)
                    # print('--------------------------------------------------------------')
                
                train_loss.backward()

                if step % exp_config.gradient_accumulation_steps == 0 or step == len(train_dataloader) - 1:
                    optimizer.step()
                    lr_scheduler.step()
                    optimizer.zero_grad() # clear the grads

                global_step += 1

if __name__ == "__main__":
    random.seed(10)
    main()





import json
import os
path_to_file = os.path.dirname(os.path.abspath(__file__))


def extract_pairs_from_txt(file):
    """
    Converts a SimulEval log file to a JSON file with specific formatting.

    Args:
        file (str): The path to the log file.

    Returns:
        pairs (list): A list of dictionaries, each containing a reference and a prediction.

    Raises:
        FileNotFoundError: If the specified log file does not exist.

    """
    pairs = []                # Holds the data that has been read in

    with open(file) as f:
        lines = f.readlines()   
        for line in lines:
            data = json.loads(line)     # Load the data from the line
            pairs.append({"prediction": data["prediction"], "reference": data["reference"]})
            # Add only prediction and reference data to the pairs list
    
    return pairs

"""
This is what the expected output should look like:
[
    {
        prediction: "prediction1",
        reference: "reference1"
    },
    {
        prediction: "prediction2",
        reference: "reference2"
    },
    {
        prediction: "prediction3",
        reference: "reference3"
    },
    ...
]
"""


import click
import json
import torch
import os
from tqdm.auto import tqdm
from typing import Optional, Dict, Sequence
import transformers
from transformers import LlamaForCausalLM, LlamaTokenizer
import time

DEFAULT_PAD_TOKEN = "[PAD]"
DEFAULT_EOS_TOKEN = "</s>"
DEFAULT_BOS_TOKEN = "</s>"
DEFAULT_UNK_TOKEN = "</s>"
MAX_SOURCE_LENGTH = 512
MAX_TARGET_LENGTH = 512
print("Max source length: ", MAX_SOURCE_LENGTH)
print("MAX target length: ", MAX_TARGET_LENGTH)


def smart_tokenizer_and_embedding_resize(
    special_tokens_dict: Dict,
    tokenizer: transformers.PreTrainedTokenizer,
):
    """Resize tokenizer and embedding.
    Note: This is the unoptimized version that may make your embedding size not be divisible by 64.
    """
    tokenizer.add_special_tokens(special_tokens_dict)
    tokenizer.add_special_tokens(
        {
            "eos_token": DEFAULT_EOS_TOKEN,
            "bos_token": DEFAULT_BOS_TOKEN,
            "unk_token": DEFAULT_UNK_TOKEN,
        }
    )


def batchify(lst, batch_size):
    for i in range(0, len(lst), batch_size):
        yield lst[i : min(i + batch_size, len(lst))]


tokenizer = LlamaTokenizer.from_pretrained(
    "decapoda-research/llama-7b-hf", model_max_length=MAX_SOURCE_LENGTH, use_fast=False
)
# enable batch inference by left padding
tokenizer.padding_side = "left"


@click.command()
@click.option("-wmt")
@click.option("-lang")
@click.option("-sys_name")
@click.option("-src_ref", help="src or ref")
@click.option("-loaded", type=bool, default=False)
@click.option("-ckpt_addr", help="LLama_finetune_april_8/checkpoint-148", default=None)
@click.option("-start_index", type=int)
@click.option("-end_index", type=int)
@click.option("-batch_size", type=int)
@click.option("-sample", type=bool)
@click.option("-num_ret", type=int)
@click.option("-task_mode", type=str, help="evaluation or machine translation")
def main(
    wmt,
    lang,
    loaded,
    src_ref,
    sys_name,
    ckpt_addr,
    start_index,
    end_index,
    batch_size,
    sample,
    num_ret,
    task_mode
):
    if not loaded:
        if lang == "zh-en":
            lang_code = "Chinese-to-English"
        elif lang == "en-de":
            lang_code = "English-to-German"
        elif lang == "en-ru":
            lang_code = "English-to-Russian"
        else:
            print("Language dir is not existed")
            exit(1)

        from mt_metrics_eval import data

        if not os.path.isdir(f"test_{wmt}_{lang}"):
            os.makedirs(f"test_{wmt}_{lang}")
            os.makedirs(f"test_{wmt}_{lang}/src")
            os.makedirs(f"test_{wmt}_{lang}/ref")
            index = ckpt_addr.split("-")[-1]
            print(index)
            os.makedirs(f"test_{wmt}_{lang}/SEScore3_output_{index}")

        evs = data.EvalSet(wmt, lang)
        mqm_scores = evs.Scores("seg", "mqm")
        print("ref: ", evs.std_ref)
        print("Annotated System: ", len(mqm_scores))

        for sys_name, score_ls in mqm_scores.items():
            assert len(score_ls) == len(evs.sys_outputs[sys_name])
            if sys_name != evs.std_ref:
                final_ref_dict = {"type": "text2score", "instances": []}
                final_src_dict = {"type": "text2score", "instances": []}
                for index, (score, output) in enumerate(
                    zip(score_ls, evs.sys_outputs[sys_name])
                ):
                    if sys_name != evs.std_ref:
                        if score != None:
                            ref = evs.sys_outputs[evs.std_ref][index]
                            cand = evs.sys_outputs[sys_name][index]
                            src = evs.src[index]
                            ref_prompt = f'You are evaluating {lang_code} Machine translation task. The correct translation is "{ref}". The model generated translation is "{cand}". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error type, major/minor label, error location of the model generated translation and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don\'t lead to loss of meaning but will be noticed.'
                            src_prompt = f'You are evaluating {lang_code} Machine translation task. The source is "{src}". The model generated translation is "{cand}". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error type, major/minor label, error location of the model generated translation and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don\'t lead to loss of meaning but will be noticed.'

                            final_ref_dict["instances"] += [
                                {"input": ref_prompt, "output": score}
                            ]
                            final_src_dict["instances"] += [
                                {"input": src_prompt, "output": score}
                            ]

            if len(final_ref_dict["instances"]) > 0:
                with open(
                    f"test_{wmt}_{lang}/ref/test_{wmt}_{lang}_{sys_name}_llama_ref_data.json",
                    "w",
                ) as f:
                    json.dump(final_ref_dict, f)

            if len(final_src_dict["instances"]) > 0:
                with open(
                    f"test_{wmt}_{lang}/src/test_{wmt}_{lang}_{sys_name}_llama_src_data.json",
                    "w",
                ) as f:
                    json.dump(final_src_dict, f)
                print(f"test_{wmt}_{lang}_{sys_name} ref and src files are saved!")
    else:
        if task_mode == 'evaluation':
            KEY_TYPE = "type"
            KEY_INSTANCES = "instances"
            # sanity check over the fields of json file
            with open(
                f"test_{wmt}_{lang}/{src_ref}/test_{wmt}_{lang}_{sys_name}_llama_{src_ref}_data.json"
            ) as fin:
                json_data = json.load(fin)
                if KEY_TYPE not in json_data.keys():
                    raise ValueError(
                        f'"{KEY_TYPE}" field must be specified for data, e.g.'
                        "{\n"
                        f'   "{KEY_TYPE}: "text2text",\n'
                        f'   "{KEY_INSTANCES}": [\n'
                        '       { "text": "Sentence 1: This is a sentence." }\n'
                        '       { "text": "Sentence 2: This is another sentence." }\n'
                        f"   ]\n"
                        "}"
                    )
            device_id = (
                torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
            )
            model = LlamaForCausalLM.from_pretrained(ckpt_addr).to(device_id)
            model.eval()

            smart_tokenizer_and_embedding_resize(
                special_tokens_dict=dict(pad_token=DEFAULT_PAD_TOKEN),
                tokenizer=tokenizer,
            )

            print("Vocab Size: ", len(tokenizer))
            print("Loaded in model and tokenizer!")

            index = ckpt_addr.split("-")[-1]
            if sample:
                save_file = open(
                    f"test_{wmt}_{lang}/SEScore3_output_sample_{index}/test_{wmt}_{lang}_{sys_name}_llama_{src_ref}_data_{start_index}_{end_index}_sample_{num_ret}.txt",
                    "w",
                )
            else:
                save_file = open(
                    f"test_{wmt}_{lang}/SEScore3_output_{index}/test_{wmt}_{lang}_{sys_name}_llama_{src_ref}_data_{start_index}_{end_index}.txt",
                    "w",
                )

            global_step = 0
            with torch.no_grad():
                with tqdm(
                    total=int(
                        len(json_data["instances"][start_index:end_index]) / batch_size
                    )
                    + 1
                ) as pbar:
                    for txts_dict in batchify(
                        json_data["instances"][start_index:end_index], batch_size
                    ):
                        batch_txts = [txt_dict["input"] for txt_dict in txts_dict]
                        inputs = tokenizer(
                            batch_txts,
                            return_tensors="pt",
                            padding=True,
                            truncation=True,
                            max_length=MAX_SOURCE_LENGTH,
                        )
                        try:
                            if sample:
                                outputs = model.generate(
                                    inputs["input_ids"].to(device_id),
                                    attention_mask=inputs["attention_mask"].to(device_id),
                                    max_new_tokens=MAX_TARGET_LENGTH,
                                    do_sample=True,
                                    top_p=0.95,
                                    temperature=0.8,
                                    num_return_sequences=num_ret,
                                )
                            else:
                                outputs = model.generate(
                                    inputs["input_ids"].to(device_id),
                                    attention_mask=inputs["attention_mask"].to(device_id),
                                    max_new_tokens=MAX_TARGET_LENGTH,
                                )
                            batch_outputs = tokenizer.batch_decode(
                                outputs,
                                skip_special_tokens=True,
                                clean_up_tokenization_spaces=True,
                            )
                            if sample:
                                for index, output in enumerate(batch_outputs):
                                    save_file.write(
                                        str(global_step + start_index)
                                        + "\t"
                                        + str(index)
                                        + "\t"
                                        + output
                                        + "[SEP_WENDA]"
                                    )
                                global_step += 1
                            else:
                                for output in batch_outputs:
                                    save_file.write(
                                        str(global_step + start_index)
                                        + "\t"
                                        + output
                                        + "[SEP_WENDA]"
                                    )
                                    global_step += 1
                        except Exception as e:
                            print(f"ERROR: {e}")
                        pbar.update(1)

        print("File is saved!")


if __name__ == "__main__":
    main()



import json
from scipy import stats

def count_loc_funct(text):
    if "Error location 5:" in text:
        return 5
    elif "Error location 4:" in text:
        return 4
    elif "Error location 3:" in text:
        return 3
    elif "Error location 2:" in text:
        return 2
    elif "Error location 1:" in text:
        return 1 
    else:
        print(text)
        print("Error!")
        exit(1)

from mt_metrics_eval import data

wmt = "wmt22"
src_ref = "ref"
lang = "zh-en"
suffix = "_0_None"

evs = data.EvalSet(wmt, lang)
mqm_scores = evs.Scores("seg", "mqm")
sys_ls = set(mqm_scores) - evs.human_sys_names # {evs.std_ref}

new_sys_ls = []
for sys in sys_ls:
    if mqm_scores[sys][0] != None:
        new_sys_ls += [sys]

sescore3_ls, gt_ls = [], []
count = 0
final_dict = {}
txt_final_dict = {}
for sys in new_sys_ls:
    final_dict[sys] = {}
    ref_data = json.load(
        open(
            f"test_{wmt}_{lang}/{src_ref}/test_{wmt}_{lang}_{sys}_llama_{src_ref}_data.json"
        )
    )
    ref_ls = [ele["output"] for ele in ref_data["instances"]]
    final_str = ""
    lines = open(
        f"test_{wmt}_{lang}/SEScore3_output_264/test_{wmt}_{sys}_llama_{src_ref}_data{suffix}.txt",
        "r",
    ).readlines()
    for ele in lines:
        final_str += ele

    scores_ls = []
    err_loc_ls = []
    cur_txt_ls = []
    for ele in final_str.split("[SEP_WENDA]")[:-1]:
        score = 0
        score += (-1) * ele.count("Major/minor: Minor")
        score += (-5) * ele.count("Major/minor: Major")
        scores_ls += [score]
        cur_txt_ls += [ele]
        
        if score != 0:
            temp_err_loc_ls = []
            for i in range(1, count_loc_funct(ele)+1):
                cur_txt = ele.split(f"Error location {i}:")[1].split('\n')[0]
                temp_err_loc_ls += [cur_txt.replace('"', '').strip()]
            err_loc_ls += [temp_err_loc_ls]
        else:
            err_loc_ls += ["no_error"]

    final_dict[sys]['err_locs'] = err_loc_ls
    final_dict[sys]['scores'] = scores_ls
    txt_final_dict[sys] = cur_txt_ls
    for sescore3, gt in zip(scores_ls, ref_ls):
        sescore3_ls += [sescore3]
        gt_ls += [gt]

res = stats.kendalltau(sescore3_ls, gt_ls)
print("Kendall: ", res[0])
res = stats.spearmanr(sescore3_ls, gt_ls)
print("Spearman: ", res[0])
res = stats.pearsonr(sescore3_ls, gt_ls)
print("Pearson: ", res[0])


import json
import os
import pandas as pd
path_to_file = os.path.dirname(os.path.abspath(__file__))


def extract_pairs_from_tsv(files: list[str]) -> list:
    """
    Extracts source-prediction-reference tuples (if available) from a .tsv file and creates a dictionairy containing those pairs

    Args:
        files (list[str]): A list of file paths to the inputted tsv files.

    Returns:
        pairs (list): A list of dictionaries, each containing a reference and a prediction.

    Raises:
        FileNotFoundError: If the specified files does not exist.

    """
    pairs = []                # Holds the data that has been read in

    print(files)
    translations = pd.read_csv(files, sep='\t')
    id_set = set()
    ref_set = set()
    src_set = set()
    for index, row in translations.iterrows():
        if row['globalSegId'] not in id_set and row['system'] == 'GPT4-5shot':
            pairs.append({'prediction': row['target'].replace("<v>", "").replace("</v>", ""), 'source': row['source']})
            id_set.add(row['globalSegId'])
            # Add only prediction and reference data to the pairs list
        if row['globalSegId'] not in ref_set and row['system'] == 'refA':
            pairs[-1]['reference'] = row['target'].replace("<v>", "").replace("</v>", "")
            ref_set.add(row['globalSegId'])
    # raise Error
    return pairs

"""
This is what the expected output should look like:
[
    {
        source: "source1",
        prediction: "prediction1",
        reference: "reference1"
    },
    {
        source: "source2",
        prediction: "prediction2",
        reference: "reference2"
    },
    {
        source: "source3",
        prediction: "prediction3",
        reference: "reference3"
    },
    ...
]
"""


from datasets import load_dataset
import random
import json
import click

@click.command()
@click.option('-src_code')
@click.option('-tar_code')
def main(src_code, tar_code):
    code_lang_dict = {'en': "English", "de": "German", "ru": "Russian"}

    if f"{tar_code}-{src_code}" == 'de-en':
        wmt = 'wmt14'
        load_addr = 'data/german_llama_ref_data.json'
        save_addr = 'data/eval_mt_german_llama.json'
    elif f"{tar_code}-{src_code}" == 'ru-en':
        wmt = 'wmt16'
        load_addr = 'data/russian_llama_ref_data.json'
        save_addr = 'data/eval_mt_russian_llama.json'
    else:
        print("Language is not supported!")

    data = load_dataset(wmt, f"{tar_code}-{src_code}")
    # extract 2.5% training data, 10% of validation data
    train_data_part1, train_data_part2, vali_data = data['train'][:10000], data['train'][10000:20000], data['validation'][:300]

    cur_data = json.load(open(load_addr))

    for cur_dict in train_data_part1['translation']:
        src_txt, tar_txt = cur_dict[src_code], cur_dict[tar_code]
        src_lang, tar_lang = code_lang_dict[src_code], code_lang_dict[tar_code]
        prompt = f"You are translating from {src_lang} to {tar_lang}. The {src_lang} text is \"{src_txt}\". The {tar_lang} text is "
        cur_data["instances"]+=[{"input": prompt, "output": tar_txt}]

    for cur_dict in train_data_part2['translation']:
        src_txt, tar_txt = cur_dict[tar_code], cur_dict[src_code]
        src_lang, tar_lang = code_lang_dict[tar_code], code_lang_dict[src_code]
        prompt = f"You are translating from {src_lang} to {tar_lang}. The {src_lang} text is \"{src_txt}\". The {tar_lang} text is "
        cur_data["instances"]+=[{"input": prompt, "output": tar_txt}]

    with open(save_addr, 'w') as f:
        json.dump(cur_data, f)

    print("File is saved!")

if __name__ == "__main__":
    main()


import glob
import json
from scipy import stats

gt_scores = open('test_caption_crowd/caption_crowd.score', 'r').readlines()
gt_scores = [float(ele[:-1].split('\t')[1]) for ele in gt_scores]

final_ls = []
for file_addr in sorted(glob.glob('test_caption_crowd/output_462/outputs/*')):
    lines = open(file_addr, 'r').readlines()
    final_ls+=lines

scores_ls = []
for ele in ''.join(final_ls).split('[SEP_WENDA]')[:-1]:
    score = 0
    score += (-1) * ele.count('Major/minor:  Minor')
    score += (-1) * ele.count('Major/minor: Minor')
    score += (-5) * ele.count('Major/minor:  Major')
    score += (-5) * ele.count('Major/minor: Major')
    if score <= -5:
        scores_ls+=[0]
    else:
        scores_ls+=[1]

res = stats.kendalltau(gt_scores, scores_ls)[0]
print("Kendall: ", res)
res = stats.spearmanr(gt_scores, scores_ls)[0]
print("Spearman: ", res)
res = stats.pearsonr(gt_scores, scores_ls)[0]
print("Pearson: ", res)
